@@ -28,11 +28,10 @@
 
 A thread is a forest or list of trees. A tree is a two element
 list where the first element is a message, and the second element
-is a possibly empty forest of replies.
-"
-  (let ((args '("show" "--format=sexp" "--format-version=1")))
+is a possibly empty forest of replies."
+  (let ((args '("show" "--format=sexp" "--format-version=4")))
     (if notmuch-show-process-crypto
-	(setq args (append args '("--decrypt"))))
+	(setq args (append args '("--decrypt=true"))))
     (setq args (append args search-terms))
     (apply #'notmuch-call-notmuch-sexp args)))
 
@@ -40,7 +39,7 @@ is a possibly empty forest of replies.
 ;; Mapping functions across collections of messages.
 
 (defun notmuch-query-map-aux  (mapper function seq)
-  "private function to do the actual mapping and flattening"
+  "Private function to do the actual mapping and flattening."
   (apply 'append
 	 (mapcar
 	   (lambda (tree)
@@ -48,20 +47,20 @@ is a possibly empty forest of replies.
 	   seq)))
 
 (defun notmuch-query-map-threads (fn threads)
-  "apply FN to every thread in  THREADS. Flatten results to a list.
+  "Apply FN to every thread in THREADS. Flatten results to a list.
 
 See the function notmuch-query-get-threads for more information."
   (notmuch-query-map-aux 'notmuch-query-map-forest fn threads))
 
 (defun notmuch-query-map-forest (fn forest)
-  "apply function to every message in a forest. Flatten results to a list.
+  "Apply function to every message in a forest. Flatten results to a list.
 
 See the function notmuch-query-get-threads for more information.
 "
   (notmuch-query-map-aux 'notmuch-query-map-tree fn forest))
 
 (defun notmuch-query-map-tree (fn tree)
-  "Apply function FN to every message in TREE. Flatten results to a list
+  "Apply function FN to every message in TREE. Flatten results to a list.
 
 See the function notmuch-query-get-threads for more information."
   (cons (funcall fn (car tree)) (notmuch-query-map-forest fn (cadr tree))))
@@ -70,7 +69,7 @@ See the function notmuch-query-get-threads for more information."
 ;; Predefined queries
 
 (defun notmuch-query-get-message-ids (&rest search-terms)
-  "Return a list of message-ids of messages that match SEARCH-TERMS"
+  "Return a list of message-ids of messages that match SEARCH-TERMS."
   (notmuch-query-map-threads
    (lambda (msg) (plist-get msg :id))
    (notmuch-query-get-threads search-terms)))
